#
# Copyright 2017-2018 (c) Andrey Galkin
#

define cfsystem::clusterssh(
    String[1]
        $namespace,
    String[1]
        $cluster,
    Boolean
        $is_primary,
    String[1]
        $peer_ipset,
    String[1]
        $user,
    String[1]
        $group = $user,
    Cfsystem::Keytype
        $key_type = 'ed25519',
    Cfsystem::Rsabits
        $key_bits = 2048, # for rsa
) {
    if $title != "${namespace}:${cluster}" {
        fail("Invalid clusterssh title = ${title}")
    }

    ensure_resource( 'cfsystem::sshdir', $user )

    $home = getparam(User[$user], 'home')
    $ssh_dir = "${home}/.ssh"
    $ssh_idkey = "${ssh_dir}/id_${cluster}"
    $persist_title = "keys:${title}"

    if $is_primary {
        $key_tune = undef
    } else {
        $cluster_info = cfsystem::query([
            'from', 'resources', ['extract', ['parameters'],
                ['and',
                    ['in', 'certname',
                        ['extract', 'certname',
                            ['select_resources',
                                ['and',
                                    ['=', 'type', 'Cfsystem::Clusterssh'],
                                    ['=', 'title', $title],
                                    ['=', ['parameter', 'is_primary'], true],
                                ]
                            ],
                        ],
                    ],
                    ['=', 'type', 'Cfsystem_persist'],
                    ['=', 'title', $persist_title],
                ]
            ],
        ])

        if size($cluster_info) != 1 {
            fail("Failed to fetch primary node info for clusterssh[${title}]")
        }

        $key_tune = $cluster_info[0]['parameters']['value']
    }

    $key_gen_opts = {
        type => $key_type,
        bits => $key_bits
    }
    $key_info = cfsystem::gen_key($title, $key_gen_opts, $key_tune)

    User[$user]
    -> file { "${ssh_dir}/config_${cluster}":
        owner   => $user,
        group   => $group,
        mode    => '0600',
        content => [
            '# Generated by CodingFuture cfsystem Puppet Module',
            '# DO NOT EDIT',
            "IdentityFile ${ssh_idkey}",
        ].join("\n")
    }

    # TODO: YES, it's insecure as the private key gets stored in catalog
    # in clear text. Need some sort of host-specific encryption.
    file { $ssh_idkey:
        owner     => $user,
        group     => $group,
        mode      => '0600',
        content   => $key_info['private'],
        show_diff => false,
    }
    file { "${ssh_idkey}.pub":
        owner     => $user,
        group     => $group,
        mode      => '0600',
        content   => "ssh-${key_info['type']} ${key_info['public']}\n",
        show_diff => false,
    }

    # TODO: options from=
    # It required hostname to be resolved to IPs due to UseDNS=no
    ssh_authorized_key { $ssh_idkey:
        ensure  => present,
        user    => $user,
        type    => $key_info['type'],
        key     => $key_info['public'],
        require => File[$ssh_dir],
    }

    cfnetwork::client_port { "any:cfssh:${namespace}_${cluster}":
        dst  => "ipset:${peer_ipset}",
        user => $user,
    }
    cfnetwork::service_port { "any:cfssh:${namespace}_${cluster}":
        src => "ipset:${peer_ipset}",
    }
}
